@@ -3,7 +3,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis-python
 #
-# Most of this work is copyright (C) 2013-2017 David R. MacIver
+# Most of this work is copyright (C) 2013-2018 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -64,7 +64,6 @@ class ConjectureData(object):
         self.status = Status.VALID
         self.frozen = False
         self.intervals_by_level = []
-        self.intervals = []
         self.interval_stack = []
         global global_test_counter
         self.testcounter = global_test_counter
@@ -76,6 +75,9 @@ class ConjectureData(object):
         self.interesting_origin = None
         self.tags = set()
         self.draw_times = []
+        self.__intervals = None
+        self.shrinking_blocks = set()
+        self.discarded = set()
 
     def __assert_not_frozen(self, name):
         if self.frozen:
@@ -147,7 +149,7 @@ class ConjectureData(object):
         self.interval_stack.append(self.index)
         self.level += 1
 
-    def stop_example(self):
+    def stop_example(self, discard=False):
         if self.frozen:
             return
         self.level -= 1
@@ -157,12 +159,36 @@ class ConjectureData(object):
         if k != self.index:
             t = (k, self.index)
             self.intervals_by_level[self.level].append(t)
-            if not self.intervals or self.intervals[-1] != t:
-                self.intervals.append(t)
+            if discard:
+                self.discarded.add(t)
 
     def note_event(self, event):
         self.events.add(event)
 
+    @property
+    def intervals(self):
+        assert self.frozen
+        if self.__intervals is None:
+            intervals = set(self.blocks)
+            for l in self.intervals_by_level:
+                intervals.update(l)
+                for i in hrange(len(l) - 1):
+                    if (
+                        l[i] not in self.discarded and
+                        l[i + 1] not in self.discarded and
+                        l[i][1] == l[i + 1][0]
+                    ):
+                        intervals.add((l[i][0], l[i + 1][1]))
+            for i in hrange(len(self.blocks) - 1):
+                intervals.add((self.blocks[i][0], self.blocks[i + 1][1]))
+            # Intervals are sorted as longest first, then by interval start.
+            self.__intervals = tuple(sorted(
+                set(intervals),
+                key=lambda se: (se[0] - se[1], se[0])
+            ))
+            del self.intervals_by_level
+        return self.__intervals
+
     def freeze(self):
         if self.frozen:
             assert isinstance(self.buffer, hbytes)
@@ -170,15 +196,6 @@ class ConjectureData(object):
         self.frozen = True
         self.finish_time = benchmark_time()
 
-        # Intervals are sorted as longest first, then by interval start.
-        for l in self.intervals_by_level:
-            for i in hrange(len(l) - 1):
-                if l[i][1] == l[i + 1][0]:
-                    self.intervals.append((l[i][0], l[i + 1][1]))
-        self.intervals = sorted(
-            set(self.intervals),
-            key=lambda se: (se[0] - se[1], se[0])
-        )
         self.buffer = hbytes(self.buffer)
         self.events = frozenset(self.events)
         del self._draw_bytes
@@ -227,7 +244,6 @@ class ConjectureData(object):
         assert len(result) == n
         assert self.index == initial
         self.buffer.extend(result)
-        self.intervals.append((initial, self.index))
 
     def draw_bytes(self, n):
         self.__assert_not_frozen('draw_bytes')
