@@ -3,7 +3,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis-python
 #
-# Most of this work is copyright (C) 2013-2017 David R. MacIver
+# Most of this work is copyright (C) 2013-2018 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -241,8 +241,8 @@ def integers(min_value=None, max_value=None):
     If min_value is not None then all values will be >= min_value. If
     max_value is not None then all values will be <= max_value
 
-    Examples from this strategy will shrink towards being positive (e.g. 1000
-    is considered simpler than -1) and then towards zero.
+    Examples from this strategy will shrink towards zero, and negative values
+    will also shrink towards positive (i.e. -n may be replaced by +n).
 
     """
 
@@ -790,8 +790,9 @@ def text(
     """Generates values of a unicode text type (unicode on python 2, str on
     python 3) with values drawn from alphabet, which should be an iterable of
     length one strings or a strategy generating such. If it is None it will
-    default to generating the full unicode range. If it is an empty collection
-    this will only generate empty strings.
+    default to generating the full unicode range (excluding surrogate
+    characters). If it is an empty collection this will only generate empty
+    strings.
 
     min_size, max_size and average_size have the usual interpretations.
 
@@ -921,16 +922,16 @@ def random_module():
     from hypothesis.control import cleanup
     import random
 
-    def seed_random(seed):
-        state = random.getstate()
-        random.seed(seed)
-        cleanup(lambda: random.setstate(state))
-        return RandomSeeder(seed)
+    class RandomModule(SearchStrategy):
+        def do_draw(self, data):
+            data.can_reproduce_example_from_repr = False
+            seed = data.draw(integers())
+            state = random.getstate()
+            random.seed(seed)
+            cleanup(lambda: random.setstate(state))
+            return RandomSeeder(seed)
 
-    return shared(
-        integers().map(seed_random),
-        'hypothesis.strategies.random_module()',
-    )
+    return shared(RandomModule(), 'hypothesis.strategies.random_module()')
 
 
 @cacheable
@@ -1621,6 +1622,7 @@ def choices():
         supports_find = False
 
         def do_draw(self, data):
+            data.can_reproduce_example_from_repr = False
             return Chooser(current_build_context(), data)
 
     return shared(
@@ -1724,6 +1726,8 @@ def data():
         supports_find = False
 
         def do_draw(self, data):
+            data.can_reproduce_example_from_repr = False
+
             if not hasattr(data, 'hypothesis_shared_data_strategy'):
                 data.hypothesis_shared_data_strategy = DataObject(data)
             return data.hypothesis_shared_data_strategy
@@ -1797,16 +1801,16 @@ def deferred(definition):
     >>> x.example()
     (((False, (True, True)), (False, True)), (True, True))
     >>> x.example()
-    (True, True)
+    True
 
     Mutual recursion also works fine:
 
     >>> a = st.deferred(lambda: st.booleans() | b)
     >>> b = st.deferred(lambda: st.tuples(a, a))
     >>> a.example()
-    (True, (True, False))
+    True
     >>> b.example()
-    (False, True)
+    (False, (False, ((False, True), False)))
 
     Examples from this strategy shrink as they normally would from the strategy
     returned by the definition.
